#include "behaviortree_ros2/bt_action_node.hpp"
#include "rbs_skill_interfaces/action/gripper_command.hpp"
#include <behaviortree_cpp/tree_node.h>
#include <behaviortree_ros2/plugins.hpp>
#include <behaviortree_ros2/ros_node_params.hpp>

using GripperCommand = rbs_skill_interfaces::action::GripperCommand;
using namespace BT;

class GripperControl : public RosActionNode<GripperCommand> {
public:
  GripperControl(const std::string &name, const NodeConfig &conf,
                 const RosNodeParams &params)
      : RosActionNode<GripperCommand>(name, conf, params) {
    if (!node_.lock()->has_parameter("open")) {
      node_.lock()->declare_parameter("open", position.open);
    }
    if (!node_.lock()->has_parameter("close")) {
      node_.lock()->declare_parameter("close", position.close);
    }
  }

  static PortsList providedPorts() {
    return providedBasicPorts({
        InputPort<std::string>("pose"),
    });
  }

  bool setGoal(RosActionNode::Goal &goal) override {
    getInput("pose", pose);
    goal.position = node_.lock()->get_parameter(pose).as_double();
    return true;
  }

  NodeStatus onResultReceived(const WrappedResult& wr) override {
    if (!wr.result->success) {
      return NodeStatus::FAILURE;
    }
    return NodeStatus::SUCCESS;
  }

private:
  struct {
    double open = 0.008;
    double close = 0.000;
  } position;

  std::string pose;
};

CreateRosNodePlugin(GripperControl, "MoveGripper");
